@@ -176,6 +176,9 @@ EXTERN char *dosemu_midi_path INIT("~/" LOCALDIR_BASE_NAME "/run/" DOSEMU_MIDI);
 /* Define this for ASPI (generic SCSI) support */
 #undef ASPI_SUPPORT
 
+/* Define this to use the X86 CPU emulator */
+#undef X86_EMULATOR
+
 /* Define this on glibc >= 2.1,
  * because though it has llseek in the library, it doesn't prototype it */
 #undef NEED_LLSEEK_PROTOTYPE
