@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -45,6 +45,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <linux/unistd.h>
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -1300,7 +1301,11 @@ static void mhp_regs(int argc, char * argv[])
 
  if (!traceloop)
   mhp_printf( "system state: %s%s%s%s\n",
+#ifdef X86_EMULATOR
+       config.cpuemu > 1 ? "emulated," : "",
+#else
        "",
+#endif
        mhpdbgc.stopped ? "stopped" : "running",
        IN_DPMI ? " in DPMI" : (in_dpmi?" in real mode while in DPMI":""),
        IN_DPMI ?(dpmi_mhp_getcsdefault()?"-32bit":"-16bit") : "");
@@ -1594,13 +1599,13 @@ static void print_log_breakpoints(void)
    num_logbp = 0;
    for (rx=0; rx <num_regex; rx++) {
      if (rxbuf[rx]) {
-       mhp_printf("log break point %d: %s\n", rx, rxpatterns[rx]);
+       mhp_printf("log breakpoint %d: %s\n", rx, rxpatterns[rx]);
        mhp_send();
        num_logbp++;
      }
    }
    if (!num_logbp) {
-     mhp_printf("no log break points aktiv\n");
+     mhp_printf("no log breakpoint active\n");
    }
 }
 
