@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -35,7 +35,6 @@
 #include "vc.h"
 #include "priv.h"
 #include "doshelpers.h"
-#include "utilities.h"
 
 #ifdef USING_NET
 #include "ipx.h"
@@ -44,6 +43,9 @@
 #include "dpmi.h"
 
 #include "keyb_server.h"
+#ifdef X86_EMULATOR
+#include "cpu-emu.h"	/* for TRACE_DPMI */
+#endif
 
 #define DJGPP_HACK	/* AV Feb 97 */
 #undef  DEBUG_INT1A
@@ -71,8 +73,7 @@ static struct timeval scr_tv;        /* For translating UNIX <-> DOS times */
 int redir_state = 0;
 
 #ifdef USE_MRP_JOYSTICK
-#error "FIXME: joystick support is broken. Sorry"
-#include "Linux/joystick.h"
+#include <linux/joystick.h>
 #include <fcntl.h>
 #include <string.h>
 static void mrp_read_joystick(void);
@@ -197,6 +198,10 @@ static void process_master_boot_record(void)
 /* returns 1 if dos_helper() handles it, 0 otherwise */
 static int dos_helper(void)
 {
+#ifdef X86_EMULATOR
+  extern void enter_cpu_emu(void);
+  extern void leave_cpu_emu(void);
+#endif
 
   switch (LO(ax)) {
   case DOS_HELPER_DOSEMU_CHECK:			/* Linux dosemu installation test */
@@ -288,12 +293,10 @@ static int dos_helper(void)
   case DOS_HELPER_SHOW_BANNER:		/* show banner */
     p_dos_str("\n\nLinux DOS emulator " VERSTR " $Date: " VERDATE " $\n");
     p_dos_str("Last configured at %s on %s\n", CONFIG_TIME, CONFIG_HOST);
-#if 0
     p_dos_str("This is work in progress.\n");
     p_dos_str("Please test against a recent version before reporting bugs and problems.\n");
     /* p_dos_str("Formerly maintained by Robert Sanders, gt8134b@prism.gatech.edu\n\n"); */
-    p_dos_str("Bugs, Patches & New Code to linux-msdos@vger.kernel.org\n\n");
-#endif
+    p_dos_str("Bugs, Patches & New Code to linux-msdos@vger.rutgers.edu\n\n");
     if (config.dpmi)
       p_dos_str("DPMI-Server Version 0.9 installed\n\n");
     break;
@@ -523,6 +526,19 @@ static int dos_helper(void)
   case DOS_HELPER_CHDIR:
         LWORD(eax) = chdir(SEG_ADR((char *), es, dx));
         break;
+#ifdef X86_EMULATOR
+  case DOS_HELPER_CPUEMUON:
+#ifdef DONT_DEBUG_BOOT
+	memcpy(&d,&d_save,sizeof(struct debug_flags));
+#endif
+	/* we could also enter from inside dpmi, provided we already
+	 * mirrored the LDT into the emu's own one */
+  	if ((config.cpuemu==1) && !in_dpmi) enter_cpu_emu();
+        break;
+  case DOS_HELPER_CPUEMUOFF:
+  	if ((config.cpuemu>1) && !in_dpmi) leave_cpu_emu();
+        break;
+#endif
     case DOS_HELPER_XCONFIG:
 #if X_GRAPHICS
 	if (config.X) {
@@ -700,7 +716,7 @@ static void int15(u_char i)
   case 0xc2:
     m_printf("PS2MOUSE: Call ax=0x%04x\n", LWORD(eax));
     if (!mice->intdrv)
-      if (mice->type != MOUSE_PS2 && mice->type != MOUSE_IMPS2) {
+      if (mice->type != MOUSE_PS2) {
 	REG(eax) = 0x0500;        /* No ps2 mouse device handler */
 	CARRY;
 	return;
@@ -1133,13 +1149,35 @@ Return: nothing
  */
 #define EMM_FILE_HANDLE 200
 
+/* uppercase and truncate to 3 letters the replacement extension */
+/* pay attention to writable strings! */
+#define ext_fix(s) { char *r=(s); \
+		     while (*r) { *r=toupper(*r); r++; } \
+		     if ((r - s) > 3) s[3]=0; }
 
 static int ms_dos(int nr)
 {
   switch (nr) {
   case 0x3d:       /* DOS handle open */
+  {
+    char *ptr = SEG_ADR((char *), ds, dx);
+
+    /* ignore explicitly selected drive by incrementing ptr by 1 */
+    if (config.emubat && !strncasecmp(ptr + 1, ":\\AUTOEXEC.BAT", 14)) {
+      ext_fix(config.emubat);
+      sprintf(ptr + 1, ":\\AUTOEXEC.%-3s", config.emubat);
+      d_printf("DISK: Substituted %s for AUTOEXEC.BAT\n", ptr + 1);
+    } else if (config.emusys && !strncmp(ptr, "\\CONFIG.SYS", 11)) {
+      ext_fix(config.emusys);
+      sprintf(ptr, "\\CONFIG.%-3s", config.emusys);
+      d_printf("DISK: Substituted %s for CONFIG.SYS\n", ptr);
+ } else if (config.emuini && !strncmp(ptr+2, "\\WINDOWS\\SYSTEM.INI", 19)) {
+ ext_fix(config.emuini);
+      sprintf(ptr+2, "\\WINDOWS\\SYSTEM.%s", config.emuini);
+      d_printf("DISK: Substituted %s for system.ini\n", ptr);
+    }
 #ifdef INTERNAL_EMS
-    if (config.ems_size && !strncmp(ptr, "EMMXXXX0", 8)) {
+    else if (config.ems_size && !strncmp(ptr, "EMMXXXX0", 8)) {
       E_printf("EMS: opened EMM file!\n");
       LWORD(eax) = EMM_FILE_HANDLE;
       NOCARRY;
@@ -1147,8 +1185,9 @@ static int ms_dos(int nr)
       return 1;
     }
 #endif
-    subst_file_ext(SEG_ADR((char *), ds, dx));
-    return 0;
+
+  return 0;
+  }
 
 #ifdef INTERNAL_EMS
   case 0x3e:       /* DOS handle close */
@@ -1605,6 +1644,9 @@ static int redir_it()
 
 /* MS-DOS */
 static void int21(u_char i) {
+#ifdef X86_EMULATOR
+  static char buf[80];
+#endif
   ds_printf("INT21 (%d) at %04x:%04x: AX=%04x, BX=%04x, CX=%04x, DX=%04x, DS=%04x, ES=%04x\n",
        redir_state, LWORD(cs), LWORD(eip),
        LWORD(eax), LWORD(ebx), LWORD(ecx), LWORD(edx), LWORD(ds), LWORD(es));
@@ -1622,6 +1664,25 @@ static void int21(u_char i) {
   }
 #endif
 
+#ifdef X86_EMULATOR
+  if ((HI(ax)==0x40) && LWORD(ecx)) {
+	char *dp = (char *)((LWORD(ds)<<4)+LWORD(edx));
+	unsigned int nb = LWORD(ecx);
+	if (nb>78) nb=78; memcpy(buf,dp,nb); buf[nb]=0;
+	ds_printf("WRITE: [%s]\n",buf);
+  }
+  else if (HI(ax)==9) {
+	char *dp = (char *)((LWORD(ds)<<4)+LWORD(edx));
+	char *q = buf;
+	int nb;
+	for (nb=0; (nb<78)&&(*dp!='$'); nb++) *q++ = *dp++;
+	buf[nb]=0;
+	ds_printf("WRITE: [%s]\n",buf);
+  }
+  else if ((HI(ax)==6) && (LO(ax)!=0xff)) {
+	ds_printf("WRITE: [%c]\n",isprint(LO(ax)? LO(ax):'.'));
+  }
+#endif
   if (!ms_dos(HI(ax)))
     default_interrupt(i);
 }
@@ -1736,10 +1797,10 @@ static void int2f(u_char i)
 
   switch (HI(ax)) {
   case 0x11:              /* redirector call? */
-    if (LO(ax) == 0x23) subst_file_ext(SEG_ADR((char *), ds, si));
-    if (mfs_redirector()) return;
+    if (mfs_redirector())
+    return;
     break;
-  
+
   case 0x16:		/* misc PM/Win functions */
     if (!config.dpmi) {
 /*  d.emu=4; */
@@ -2003,16 +2064,19 @@ void do_int(int i)
 		} else {
 			clear_IF();
 		}
+/* D_printf("DPMI: do_int: dpmi_eflags=%08x\n",dpmi_eflags); */
 	}
 	
+#ifndef TRACE_DPMI
  	if ((d.defint > 2) && (((i != 0x28) && (i != 0x2f)) || in_dpmi)) {
- 		di_printf("Do INT0x%02x eax=0x%08x ebx=0x%08x ss=0x%08x esp=0x%08x\n"
- 			  "           ecx=0x%08x edx=0x%08x ds=0x%08x  cs=0x%08x ip=0x%08x\n"
- 			  "           esi=0x%08x edi=0x%08x es=0x%08x flg=0x%08x\n",
+ 		di_printf("Do INT0x%02x eax=0x%08x ebx=0x%08x ss=0x%04x esp=0x%08x\n"
+ 			  "           ecx=0x%08x edx=0x%08x ds=0x%04x  cs=0x%04x ip=0x%04x\n"
+ 			  "           esi=0x%08x edi=0x%08x es=0x%04x flg=0x%08x\n",
  			  i, _EAX, _EBX, _SS, _ESP,
  			  _ECX, _EDX, _DS, _CS, _IP,
  			  _ESI, _EDI, _ES, (int) read_EFLAGS());
  	}
+#endif
 	
 #if 1  /* This test really ought to be in the main loop before
  	*  instruction execution not here. --EB 10 March 1997 
