@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -212,51 +212,3 @@ void rtc_init (void)
   SET_CMOS(CMOS_SECALRM,  0);
   UNLOCK_CMOS;
 }
-
-
-#ifdef USE_THREADS
-
-struct lock_struct *resource_cmos = NULL;
-
-void background_onesec_thread(int start)
-{
-  static hitimer_t cnt1000;
-  static unsigned long dt1000;
-  static int cnt600s = 0;
-  static int first=1;
-
-  if (!start) {
-    /* we come here _before_ any thread is started
-     * but threading is active and we run at scope of thread0.
-     * We now create the resources we need.
-     */
-     resource_cmos = create_resource("cmos");
-     cmos_init();
-     cmos_reset();
-     cnt600s = 0;
-     first = 1;
-     return;
-  }
-  /* here we are a separate thread, real-time */
-  while (1) {
-    if (first) {
-      rtc_init();
-      cnt1000 = GETusSYSTIME();
-      dt1000 = 1000000;
-      first = 0;
-    }
-    thread_usleep(dt1000);
-    if ((++cnt600s) >= 600) {	/* every 10min */
-      cnt600s = 0;
-      (void)sys_time_calib();
-    }
-    cnt1000 += 1000000;		/* expected time */
-
-    rtc_update();
-    dt1000 = 1000000 - (long)(GETusSYSTIME()-cnt1000);
-  }
-}
-
-#endif	/*USE_THREADS*/
-
-
