@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -23,12 +23,16 @@
 #include "keymaps.h"
 #include "memory.h"
 #include "bios.h"
+#include "kversion.h"
 #include "lpt.h"
 #include "int.h"
 
 #include "dos2linux.h"
 #include "priv.h"
 #include "utilities.h"
+#ifdef X86_EMULATOR
+#include "cpu-emu.h"
+#endif
 #include "mhpdbg.h"
 
 
@@ -107,6 +111,7 @@ config_defaults(void)
     config.rdtsc = 0;
     config.mathco = 0;
     config.smp = 0;
+    config.CPUSpeedInMhz = 150;	/* instruction cycles per us, entry level */
 
     open_proc_scan("/proc/cpuinfo");
     switch (get_proc_intvalue_by_key(
@@ -121,22 +126,37 @@ config_defaults(void)
         if (cpuflags && strstr(cpuflags, "tsc")) {
           /* bogospeed currently returns 0; should it deny
            * pentium features, fall back into 486 case */
+#if LX_KERNEL_VERSION > 2001126
 	  if ((kernel_version_code > 0x20100+126)
 	       && (cpuflags = get_proc_string_by_key("cpu MHz"))) {
 	    int di,df;
+	    /* last known proc/cpuinfo format is xxx.xxxxxx, with 3
+	     * int and 6 decimal digits - but what if there are less
+	     * or more digits??? */
 	    if (sscanf(cpuflags,"%d.%d",&di,&df)==2) {
+		char cdd[8]; int i;
+		long long chz = 0;
+		char *p = cpuflags;
+		while (*p!='.') p++; p++;
+		for (i=0; i<6; i++) cdd[i]=(*p && isdigit(*p)? *p++:'0');
+		cdd[6]=0; sscanf(cdd,"%d",&df);
 		/* speed division factor to get 1us from CPU clocks - for
 		 * details on fast division see timers.h */
-		long long chz = 0;		/* strange gcc warning */
 		chz = (di * 1000000) + df;
+
+		/* speed division factor to get 1us from CPU clock */
 		config.cpu_spd = (LLF_US*1000000)/chz;
 
-		/* speed division factor to get 838ns from CPU clocks */
+		/* speed division factor to get 838ns from CPU clock */
 		config.cpu_tick_spd = (LLF_TICKS*1000000)/chz;
 
 		fprintf (stderr,"kernel CPU speed is %Ld Hz\n",chz);
 /*		fprintf (stderr,"CPU speed factors %ld,%ld\n",
 			config.cpu_spd, config.cpu_tick_spd); */
+		config.CPUSpeedInMhz = di + (df>500000);
+#ifdef X86_EMULATOR
+		fprintf (stderr,"CPU-EMU speed is %d MHz\n",config.CPUSpeedInMhz);
+#endif
 		break;
 	    }
 	    else
@@ -145,6 +165,9 @@ config_defaults(void)
 	  else
 	    cpuflags=0;
 	  if (!cpuflags) {
+#else
+	  if (1) {
+#endif
             if (!bogospeed(&config.cpu_spd, &config.cpu_tick_spd)) {
               break;
             }
@@ -172,6 +195,9 @@ config_defaults(void)
 
     config.hdiskboot = 1;	/* default hard disk boot */
     config.mappingdriver = 0;
+#ifdef X86_EMULATOR
+    config.cpuemu = 0;
+#endif
     config.mem_size = 640;
     config.ems_size = 0;
     config.ems_frame = 0xd000;
@@ -344,6 +370,10 @@ void dump_config_status(void *printfunc)
 
     (*print)("pci %d\nrdtsc %d\nmathco %d\nsmp %d\n",
                  config.pci, config.rdtsc, config.mathco, config.smp);
+    (*print)("cpuspeed %d\n", config.CPUSpeedInMhz);
+#ifdef X86_EMULATOR
+    (*print)("cpuemu %d\n", config.cpuemu);
+#endif
 
     (*print)("mappingdriver %s\n", config.mappingdriver ? config.mappingdriver : "auto");
     (*print)("hdiskboot %d\nmem_size %d\n",
@@ -589,90 +619,6 @@ static void restore_usedoptions(char *usedoptions)
     }
 }
 
-static int find_option(char *option, int argc, char **argv)
-{
-  int i;
-  if (argc <=1 ) return 0;
-  for (i=1; i < argc; i++) {
-    if (!strcmp(argv[i], option)) {
-      return i;
-    }
-  }
-  return 0;
-}
-
-static int option_delete(int option, int *argc, char **argv)
-{
-  int i;
-  if (option >= *argc) return 0;
-  for (i=option; i < *argc; i++) {
-    argv[i] = argv[i+1];
-  }
-  *argc -= 1;
-  return option;
-}
-
-void secure_option_preparse(int *argc, char **argv)
-{
-  PRIV_SAVE_AREA
-  char *opt;
-  int runningsuid = get_orig_uid() != get_orig_euid();
-
-  char * get_option(char *key, int with_arg)
-  {
-    char *p;
-    int o = find_option(key, *argc, argv);
-    if (!o) return 0;
-    o = option_delete(o, argc, argv);
-    if (!with_arg) return "";
-    if (!with_arg || o >= *argc) return "";
-    if (argv[o][0] == '-') {
-      usage();
-      exit(0);
-    }
-    p = strdup(argv[o]);
-    option_delete(o, argc, argv);
-    return p;
-  }
-
-  if (runningsuid) unsetenv("DOSEMU_LAX_CHECKING");
-  else setenv("DOSEMU_LAX_CHECKING", "on", 1);
-
-  if (*argc <=1 ) return;
-  enter_priv_off();
-                                                  
-  opt = get_option("--Fusers", 1);
-  if (opt && opt[0]) {
-    if (runningsuid) {
-      fprintf(stderr, "Bypassing /etc/dosemu.users not allowed for suid-root\n");
-      exit(0);
-    }
-    DOSEMU_USERS_FILE = opt;
-  }
-
-  opt = get_option("--Flibdir", 1);
-  if (opt && opt[0]) {
-    if (runningsuid) {
-      fprintf(stderr, "Bypassing systemwide configuration not allowed for suid-root\n");
-      exit(0);
-    }
-    DOSEMU_LIB_DIR = opt;
-    DOSEMU_USERS_FILE = "none";
-  }
-
-  opt = get_option("--Fimagedir", 1);
-  if (opt && opt[0]) {
-    if (runningsuid) {
-      fprintf(stderr, "Bypassing systemwide boot path not allowed for suid-root\n");
-      exit(0);
-    }
-    DOSEMU_HDIMAGE_DIR = opt;
-  }
-
-  leave_priv_setting();
-}
-
-
 /*
  * DANG_BEGIN_FUNCTION config_init
  * 
@@ -1037,6 +983,12 @@ config_init(int argc, char **argv)
     c_printf(" uid=%d (cached %d) gid=%d (cached %d)\n",
         geteuid(), get_cur_euid(), getegid(), get_cur_egid());
 
+#ifdef X86_EMULATOR
+    if (config.cpuemu && config.speaker==SPKR_NATIVE) {
+	c_printf("SPEAKER: can`t use native mode with cpu-emu\n");
+	config.speaker=SPKR_EMULATED;
+    }
+#endif
     if (config_check_only) {
 	dump_config_status(0);
 	usage();
@@ -1097,10 +1049,12 @@ int parse_debugflags(const char *s, unsigned char flag)
     char            c;
     int ret = 0;
 #ifdef X_SUPPORT
-    const char      allopts[] = "dARWDCvXkiTsm#pQgcwhIExMnPrSeZ";
+    const char      allopts[] = "dARWDCvXkiTtsm#pQgcwhIExMnPrSeZ";
 #else
-    const char      allopts[] = "dARWDCvkiTsm#pQgcwhIExMnPrSeZ";
+    const char      allopts[] = "dARWDCvkiTtsm#pQgcwhIExMnPrSeZ";
 #endif
+/*    abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ  */
+/*      *** *** * ** * **  ***  * ***   *   *  *****  ** *  */
 
     /*
      * if you add new classes of debug messages, make sure to add the
@@ -1214,6 +1168,14 @@ int parse_debugflags(const char *s, unsigned char flag)
 	case 'Z':
 	    d.pci = flag;       /* PCI */
 	    break;
+#ifdef X86_EMULATOR
+	case 'e':		/* cpu-emu */
+	    d.emu = flag;
+	    break;
+	case 't':		/* dpmi */
+	    d.dpmit = flag;
+	    break;
+#endif
 	case 'a':{		/* turn all on/off depending on flag */
 		char           *newopts = (char *) malloc(strlen(allopts) + 2);
 
