@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -18,7 +18,7 @@
 #include "keyb_clients.h"
 #include "termio.h"
 
-#define KBBUF_SIZE 80
+#define KBBUF_SIZE (KEYB_QUEUE_LENGTH / 2)
 
 /* LED FLAGS (from Linux keyboard code) */
 #define LED_SCRLOCK	0
