@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -148,8 +148,8 @@ static unsigned char *recode(unsigned int *out, unsigned char *in)
           *(out++) = 0;
           return in;
         }
-        /* default:
-           fall through */
+        default:
+        /* fall through */
       }
     }
     default: {
