@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -46,8 +46,12 @@
 #include <sys/stat.h>
 #include <errno.h>
 #ifdef __linux__
+#if GLIBC_VERSION_CODE < 1000
+#include <linux/fs.h>
+#else
 #include <limits.h>
 #endif
+#endif
 #include <pwd.h>
 
 #include "config.h"
