@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -29,13 +29,6 @@ static struct vm86_regs VIDEO_SAVED_REGS;
 static struct vm86_regs INT15_SAVED_REGS;
 #define S_REG(reg) (SAVED_REGS.##reg)
 #endif
-#ifdef __NetBSD__
-static struct sigcontext SAVED_REGS;
-static struct sigcontext MOUSE_SAVED_REGS;
-static struct sigcontext VIDEO_SAVED_REGS;
-static struct sigcontext INT15_SAVED_REGS;
-#define S_REG(reg) (SAVED_REGS.##reg)
-#endif
 
 
 /* these are used like:  S_LO(ax) = 2 (sets al to 2) */
@@ -1433,12 +1426,13 @@ decode_modify_segreg_insn(struct sigcontext_struct *scp, unsigned
 	    decode_use_16bit = 1;
     }
 	
+    /* first try mov sreg, .. (equal for 16/32 bit operand size) */
+    if ((len = decode_8e(scp, segment, sreg))) {
+      _eip = old_eip;
+      return len + size_prfix;
+    }
+ 
     if (decode_use_16bit) {	/*  32bit decode not implemented yet */
-      /* first try mov sreg, .. */
-      if ((len = decode_8e(scp, segment, sreg))) {
-	_eip = old_eip;
-	return len + size_prfix;
-      }
       /* then try lds, les ... */
       if ((len = decode_load_descriptor(scp, segment, sreg))) {
 	_eip = old_eip;
@@ -1480,7 +1474,7 @@ decode_modify_segreg_insn(struct sigcontext_struct *scp, unsigned
 	case 0xa9:		/* pop gs */
 	    len = 2;
 	    *segment = *ssp;
-	    *sreg = FS_INDEX;
+	    *sreg = GS_INDEX;
 	    _esp += decode_use_16bit ? 2 : 4;
 	break;
 	}
