@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -42,11 +42,14 @@
 #include <string.h>
 #include <errno.h>
 #include <sys/time.h>
-#include <netinet/in.h>
 #if GLIBC_VERSION_CODE >= 2000
+  #if GLIBC_VERSION_CODE == 2000
+    #include <asm/byteorder.h>
+  #endif
   #include <netinet/if_ether.h>
 #else
-  #include "Linux/if_ether.h"
+  #include <asm/byteorder.h>
+  #include <linux/if_ether.h>
 #endif
 #include "libpacket.h"
 #include "dosnet.h"
