@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -79,12 +79,13 @@ __asm__("___START___: jmp _emulate\n");
 #ifdef __linux__
 #if GLIBC_VERSION_CODE >= 2000
 #include <sys/vt.h>
-#else
-#include "Linux/vt.h"
-#endif
 #include <sys/kd.h>
-#include "Linux/fd.h"
-#include "Linux/hdreg.h"
+#else
+#include <linux/vt.h>
+#include <linux/kd.h>
+#endif
+#include <linux/fd.h>
+#include <linux/hdreg.h>
 #include <sys/vm86.h>
 #include <syscall.h>
 #endif
@@ -127,6 +128,9 @@ __asm__("___START___: jmp _emulate\n");
 #ifdef USE_SBEMU
 #include "sound.h"
 #endif
+#ifdef X86_EMULATOR
+#include "cpu-emu.h"
+#endif
 
 extern void     stdio_init(void);
 extern void     time_setting_init(void);
@@ -320,7 +324,7 @@ static void do_liability_disclaimer_prompt(void)
   char buf[32];
   char disclaimer_file_name[256];
   static char text[] ="
-  The Linux DOSEMU, Copyright (C) 2001 the 'DOSEMU-Development-Team'.
+  The Linux DOSEMU, Copyright (C) 2000 the 'DOSEMU-Development-Team'.
   This program is  distributed  in  the  hope that it will be useful,
   but  WITHOUT  ANY  WARRANTY;   without even the implied warranty of
   MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the file
@@ -382,7 +386,6 @@ emulate(int argc, char **argv)
 #endif
 {
     extern void parse_dosemu_users(void);
-    extern void secure_option_preparse(int *argc, char **argv);
     int e;
 
     srand(time(NULL));
@@ -400,12 +403,6 @@ emulate(int argc, char **argv)
     /* NOW! it is safe to touch the priv code.  */
     priv_init();  /* This must come first! */
 
-    /* Before we even try to give options to the parser,
-     * we pre-filter some dangerous options and delete them
-     * from the arguments list
-     */
-    secure_option_preparse(&argc, argv);
-
     /* This has to come next:
      * Parse dosemu.users _before_ any argument usage to catch
      * a not allowed user playing with overruns and kick him out
@@ -428,9 +425,11 @@ emulate(int argc, char **argv)
     port_init();		/* setup port structures, before config! */
     version_init();		/* Check the OS version */
     config_init(argc, argv);	/* parse the commands & config file(s) */
+#ifdef X86_EMULATOR
 #ifdef DONT_DEBUG_BOOT		/* cpuemu only */
     memcpy(&d_save,&d,sizeof(struct debug_flags));
     if (d.emu) memset(&d,0,sizeof(struct debug_flags));
+#endif
 #endif
     get_time_init();
     stdio_init();		/* initialize stdio & open debug file */
@@ -519,7 +518,7 @@ ign_sigs(int sig)
     error("signal %d received in leavedos()\n", sig);
     show_regs(__FILE__, __LINE__);
     flush_log();
-    if (sig == SIG_TIME)
+    if (sig == SIGALRM)
 	timerints++;
     else
 	otherints++;
@@ -568,10 +567,14 @@ leavedos(int sig)
 
     itv.it_interval.tv_sec = itv.it_interval.tv_usec = 0;
     itv.it_value = itv.it_interval;
-    if (setitimer(TIMER_TIME, &itv, NULL) == -1) {
+    if (setitimer(ITIMER_REAL, &itv, NULL) == -1) {
 	g_printf("can't turn off timer at shutdown: %s\n", strerror(errno));
     }
-    SETSIG(SIG_TIME, ign_sigs);
+    SETSIG(SIGALRM, ign_sigs);
+#ifdef X86_EMULATOR
+    setitimer(ITIMER_PROF, &itv, NULL);
+    SETSIG(SIGPROF, ign_sigs);
+#endif
     SETSIG(SIGSEGV, ign_sigs);
     SETSIG(SIGILL, ign_sigs);
     SETSIG(SIGFPE, ign_sigs);
@@ -584,7 +587,7 @@ leavedos(int sig)
       g_printf("SPEAKER: sound off\n");
       speaker_off();		/* turn off any sound */
     }
-    if (config.speaker==SPKR_NATIVE) {
+    else if (config.speaker==SPKR_NATIVE) {
        g_printf("SPEAKER: sound off\n");
        /* Since the speaker is native hardware use port manipulation,
 	* we don't know what is actually implementing the kernel's
@@ -605,6 +608,12 @@ leavedos(int sig)
     keyb_server_close();
     keyb_client_close();
 
+#if defined(X86_EMULATOR)
+    /* if we are here with config.cpuemu>1 something went wrong... */
+    if (config.cpuemu>1) {
+    	leave_cpu_emu();
+    }
+#endif
     show_ints(0, 0x33);
     g_printf("calling disk_close_all\n");
     disk_close_all();
