@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -10,9 +10,6 @@
 #include <sys/vt.h>
 #include <sys/kd.h>
 #endif
-#ifdef __NetBSD__
-#include <machine/pcvt_ioctl.h>
-#endif
 #include <errno.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -35,10 +32,6 @@ static struct stat orig_stat; /* original info of the VT */
 
 /* One of these has to work */
 static const char * CONSOLE[] =   { 
-#ifdef __NetBSD__
-  "/dev/console",
-  "/dev/ttyv0",
-#endif
 #ifdef __linux__
   "/dev/console",
   "/dev/tty0",
@@ -76,9 +69,6 @@ unsigned short detach (void) {
   
 #ifdef __linux__
   struct vt_stat vts;
-#endif
-#ifdef __NetBSD__
-  int active;
 #endif
   int pid;
   int fd;
@@ -89,12 +79,6 @@ unsigned short detach (void) {
     return(0);
   }
   
-#ifdef __NetBSD__
-  if (ioctl(fd, VT_GETACTIVE, &active) < 0) {
-    perror("VT_GETSTATE");
-    return(0);
-  }
-#endif
 #ifdef __linux__
   if (ioctl(fd, VT_GETSTATE, &vts) < 0) {
     perror("VT_GETSTATE");
@@ -158,9 +142,6 @@ unsigned short detach (void) {
   fchmod (0, S_IRUSR | S_IWUSR);
 
   setsid();
-#ifdef __NetBSD__
-  return(active);			/* return old VT. */
-#endif
 #ifdef __linux__
   return(vts.v_active); /* return old VT. */
 #endif
@@ -194,10 +175,6 @@ void restore_vt (unsigned short vt) {
 
 }
 
-#ifdef __NetBSD__
-#undef VT_DISALLOCATE			/* XXX */
-#endif
-
 /* its not really critical if this succeeds */
 void disallocate_vt (void) {
 #ifdef VT_DISALLOCATE /* only use for >1.1.54 */
