@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -34,7 +34,7 @@
 #ifndef	_BASE_H
 #define	_BASE_H
 
-#if !defined(__linux__) && !defined(__NetBSD__)
+#if !defined(__linux__)
 
 #include <mach.h>
 #include <mach_error.h>
@@ -326,7 +326,7 @@ typedef int onoff_t;
 #define VGA_ENABLED		1
 #define VGA_DISABLED		2
 
-#if !defined(__linux__) && !defined(__NetBSD__)
+#if !defined(__linux__)
 #include <mach/thread_status.h>
 #include <machine/psl.h>
 #include <i386/pio.h>
@@ -395,17 +395,12 @@ typedef struct i386_thread_state state_t;
 #include "cpu.h"
 #endif
 
-#ifdef __NetBSD__
-typedef struct sigcontext state_t;
-#endif
 #ifdef __linux__
 typedef struct vm86_regs state_t;
 #endif
 typedef int boolean_t;
 typedef caddr_t vm_address_t;
-#ifndef __NetBSD__
 typedef size_t vm_size_t;
-#endif
 
 
 /* this is used for memory objects (pointer to malloc()ed memory */
@@ -432,7 +427,7 @@ typedef size_t vm_size_t;
 
 EXTERN FILE *dbg_fd INIT(NULL);
 
-#if !defined(__linux__) && !defined(__NetBSD__)
+#if !defined(__linux__)
 
 #define MAX_IO_PORTS	128
 extern u_char io_ports[MAX_IO_PORTS];
