@@ -261,7 +261,8 @@ EXTERN int v_8514_base INIT(0);
 #define CIRRUS		7
 #define MATROX		8
 #define WDVGA		9
-#define MAX_CARDTYPE	WDVGA
+#define SIS		10
+#define MAX_CARDTYPE	SIS
 
 EXTERN void get_screen_size (void);
 EXTERN void set_video_bios_size(void);
